\name{SwitchingVolatility-package}
\alias{SwitchingVolatility-package}
\alias{SwitchingVolatility}
\docType{package}
\title{
Investigate normal Hidden Markov Models with switching volatility
}
\description{
We assume a Hidden Markov Model (HMM) to be the model of a given time series. The components are normally distributed with mean=0 and state dependent standard deviations. \code{\link{em_test_normal_sigma}} tests the Hypothesis of m_0  regimes of the HMM (the fuction \code{\link{calc_weights}} is a help function to calculate the weights of the asymptotic chi^2 mixture of the test statistic).

\code{\link{norm.HMM.generate_sample}} generates a realization af a HMM for given parameters. The function \code{\link{norm.HMM.mle}} estimates the parameters for a given dataset by (full) mle. Given a dataset and (estimated) parameters the function \code{\link{norm.HMM.forecast.prop}} calculates the distribution of (X_{t+h}|X_t,X_{t-1},...). Further the function  \code{\link{norm.HMM.viterbi}} implements the Viterbi algorithm to compute the most likely sequence of states for given parameters and observations.

Finally the functions are applied to a real daily stock log-return dataset (see examples below and \code{\link{stock_returns}}). For computing log-returns we provide the function \code{\link{price2logreturn}}.

}
\details{
\tabular{ll}{
Package: \tab SwitchingVolatility\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2010-11-23\cr
License: \tab GPL-2 \cr
LazyLoad: \tab yes\cr
}

}
\author{
Joern Dannemann, Hajo Holzmann, Florian Ketterer, Florian Schwaiger

Maintainer: Florian Schwaiger <schwaige@mathematik.uni-marburg.de>}
\references{
Walter Zucchini: Hidden Markov Models for Time Series

Li, P. and Chen, J. (2009): Testing the order of a Finite mixture, Journal of the American Statistical Association. ahead of print.
}
\examples{
#-----------------------------------------------------------#
#Fit: HMM's, sdf: normal distribution (as scale family)     #
#EXAMPLE                                                    #
#-----------------------------------------------------------#

data(stock_returns)
x=stock_returns

split.screen(c(1,2))
plot.ts(x)
abline("h"=mean(x),lty=2,col=2)
screen(2)
plot(density(x),main="")

mean(x)
x=x-mean(x)

#----------------------------------#
#             1. STOP              #
#----------------------------------#

dev.off()

#testing the hypothesis of 1 regimes
a1 = em_test_normal_sigma(dataset=x, m0=1)
a1 #p-value is zero, so we reject the hypothesis


#testing the hypothesis of 2 regimes
a2 = em_test_normal_sigma(dataset=x, m0=2)
a2 #small p-value, so we reject the hypothesis

#testing the hypothesis of 3 regimes
a3 = em_test_normal_sigma(dataset=x, m0=3)
a3 #we maintain the hypothesis

plot(a3)


#----------------------------------#
#             2. STOP              #
#----------------------------------#

#now we fit the parameters of the HMM with the full likelihood
m0=3
sigma0 = a3@mle_hyp[4:6]
HMM=norm.HMM.mle(x=x,m=m0,sigma0=sigma0)
round(HMM$sigma,digits=3) #without permuting the states s.t. sigma_1<sigma_2<sigma_3
round(HMM$gamma,digits=3)

#----------------------------------#
#             3. STOP              #
#----------------------------------#


#after permuting the states
p.hat.sigma=sort(HMM$sigma)
p.hat.sigma
HILF=HMM$gamma
hilf=rank(HMM$sigma)
HILF1=HMM$gamma[hilf,]
HILF2=HILF1[,hilf]
p.hat.gamma=HILF2
round(p.hat.gamma,6)


#we fit the most likely sequence of states, given the parameters via Viterbi algorithm 
estimate.states=norm.HMM.viterbi(x,m=3,HMM$sigma,HMM$gamma)
estimate.states


#----------------------------------#
#             4. STOP              #
#----------------------------------#


#permute the states s.t. sigma_1<sigma_2<sigma_3
hilf=rank(HMM$sigma) 
p.estimate.states=hilf[estimate.states]

y=x
s=p.estimate.states
par(fig=c(0,1,0.3,1))
par(las=1)
par(mar=c(0,5,2,2))
plot.ts(y,xlab="",xaxt="n",ylab="log Returns") 
par(fig=c(0,1,0,0.3),new=TRUE)
par(mar=c(5,5,0,2))
plot(1:length(y),s,col=2,type="h",yaxt="n",xlab="",ylim=c(0,5),ylab="state")
axis(2,at=c(1,2,3),labels=c(1,2,3))


#----------------------------------#
#             5. STOP              #
#----------------------------------#


#simulate with estimated parameters
m0=3
n0=1000
p.hat.sigma #estimators after permuting the states s.t. sigma_1<sigma_2<sigma_3
p.hat.gamma

simHMM=norm.HMM.generate_sample(n=n0,m=m0,p.hat.sigma,p.hat.gamma)
simHMM$x
simHMM$s

y=simHMM$x
s=simHMM$s
par(fig=c(0,1,0.3,1))
par(las=1)
par(mar=c(0,5,2,2))
plot.ts(y,xlab="",xaxt="n",ylab="Simulated log Returns") 
par(fig=c(0,1,0,0.3),new=TRUE)
par(mar=c(5,5,0,2))
plot(1:length(y),s,col=2,type="h",yaxt="n",xlab="",ylim=c(0,5),ylab="state")
axis(2,at=c(1,2,3),labels=c(1,2,3))


#----------------------------------#
#             6. STOP              #
#----------------------------------#

#---------------------
#Forecast Distribution
#---------------------

#----------------------------------------------------------------#
#comparison of forecast distribution and marginal distribution   #
#->should be equal if time horizon tends to infinity             #
#----------------------------------------------------------------#

dev.off()
FORECAST.HMM=norm.HMM.forecast.prop(x,m=3,sigma=HMM$sigma,gamma=HMM$gamma,H=1000)

alpha = HMM$delta

sdeviation= HMM$sigma

horizon=100
x.pred=seq(-10,10,length=horizon)
for(i in 1:horizon){
 alpha1=FORECAST.HMM$xi[,i]
 y=alpha[1]*dnorm(x.pred,sd=sdeviation[1])+alpha[2]*dnorm(x.pred,sd=sdeviation[2])+alpha[3]*dnorm(x.pred,sd=sdeviation[3])
 plot(x.pred,y,type="l",main=paste("Horizon:",i))
 yy=alpha1[1]*dnorm(x.pred,sd=sdeviation[1])+alpha1[2]*dnorm(x.pred,sd=sdeviation[2])+alpha1[3]*dnorm(x.pred,sd=sdeviation[3])
 lines(x.pred,yy,col=2,lty=3)
 Sys.sleep(0.1)
}


}
